
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Server.Kestrel.Core;
using System.Configuration;
using System.Net;
using System.Net.Http;
using System.Net.Sockets;
using System.Text;
using WCS.BLL.Config;
using WCS.BLL.DbModels;
using WCS.BLL.Manager;
using WCS.BLL.Services.IService;
using WCS.BLL.Services.Service;
using WCS.WebApi;
using WCS.WebApi.Controllers;

namespace WebApi
{
    public class Program
    {
        public static void Main(string[] args)
        {

            WebSoceketManager.InitWebSocket();

            DbInit.InitDb();

            LocalFile.SaveConfig();

            TCPClientManager.InitTcpClient();

            ShelfManager.InitShelves();

            //λƺͰ
            TCPClientManager.InitStatus();

            WarningManager.StartWarningMessageThread();

            var builder = WebApplication.CreateBuilder(args);

            // Add services to the container.
            builder.Services.AddControllers();
            // Learn more about configuring Swagger/OpenAPI at https://aka.ms/aspnetcore/swashbuckle
            builder.Services.AddEndpointsApiExplorer();
            builder.Services.AddSwaggerGen();

            builder.Services.AddScoped<IInstoreService, InstoreService>();
            builder.Services.AddScoped<IOutstoreService, OutstoreService>();
            builder.Services.AddScoped<IHomerService, HomerService>();
            builder.Services.AddScoped<IUserService, UserService>();
            builder.Services.AddScoped<IInterfaceRecordService, InterfaceRecordService>();
            builder.Services.AddScoped<IMatBaseInfoService, MatBaseInfoService>();
            builder.Services.AddScoped<IMatInventoryDetailService, MatInventoryDetailService>();
            builder.Services.AddScoped<IStoreInfoService, StoreInfoService>();
            builder.Services.AddScoped<IStockTakingService, StockTakingService>();
            builder.Services.AddScoped<ISelfCheckService, SelfCheckService>();
            builder.Services.AddScoped<IWarningService, WarningService>();
            builder.Services.AddScoped<IInOutRecordService, InOutRecordService>();
            //롢ɵõģʽ
            builder.Services.AddSingleton<IGenerateService, GenerateService>();

            var app = builder.Build();

            app.UseMiddleware<RequestResponseLoggingMiddleware>();
            // Configure the HTTP request pipeline.
            if (app.Environment.IsDevelopment())
            {
                app.UseSwagger();
                app.UseSwaggerUI();
            }

            app.UseAuthorization();
            app.MapControllers();
            app.Run("http://+:8888");
        }
    }
}
