﻿using HandyControl.Controls;
using MiniExcelLibs;
using Ping9719.WpfEx.Mvvm;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Input;
using 货架标准上位机.Views.Controls;
using 货架标准上位机.Api;
using WCS.Model;
using WCS.Model.ApiModel.Home;
using WCS.Model.ApiModel.StoreInfo;
using WCS.BLL.DbModels;
using WCS.Model.ApiModel.MatBaseInfo;
using WCS.Model.ApiModel.User;
using WCS.Model.ApiModel;
using Newtonsoft.Json.Bson;

namespace 货架标准上位机.ViewModel
{
    public class ShelfInfoViewModel : BindableBase
    {
        public ShelfInfoViewModel()
        {

        }

        #region Property
        private List<ShelfInfoModel> dataGridItemSource;
        public List<ShelfInfoModel> DataGridItemSource
        {
            get { return dataGridItemSource; }
            set
            {
                SetProperty(ref dataGridItemSource, value);
            }
        }

        private ShelfInfoModel selectedataGridItem;
        public ShelfInfoModel SelectedataGridItem
        {
            get { return selectedataGridItem; }
            set
            {
                SetProperty(ref selectedataGridItem, value);
            }
        }


        /// <summary>
        /// 物料批次
        /// </summary>
        private string shelfCode;
        public string ShelfCode
        {
            get { return shelfCode; }
            set
            {
                SetProperty(ref shelfCode, value);
            }
        }

        private List<ShelfTypeModel> shelfTypeItems;
        public List<ShelfTypeModel> ShelfTypeItems
        {
            get { return shelfTypeItems; }
            set
            {
                SetProperty(ref shelfTypeItems, value);
            }
        }
        public void InitShelfTypeItems()
        {
            //调用接口更新！
            Task.Run(() =>
            {
                var body = new RequestBase()
                {
                    UserName = LocalStatic.CurrentUser,
                    DeviceType = LocalFile.Config.DeviceType,
                };

                var Result = ApiHelp.GetDataFromHttp<PageQueryResponse<ShelfTypeModel>>(LocalFile.Config.ApiIpHost + "home/getShelfTypes", body, "POST");
                if (Result != null && Result.Data != null && Result.Data.Lists.Count() > 0)
                {
                    ShelfTypeItems = Result.Data.Lists;
                }
            });
        }

        private ShelfTypeModel selectedShelfTypeItem;
        public ShelfTypeModel SelectedShelfTypeItem
        {
            get { return selectedShelfTypeItem; }
            set
            {
                SetProperty(ref selectedShelfTypeItem, value);
            }
        }
        #endregion

        #region Command
        public ICommand BtnResetCommand { get => new DelegateCommand(BtnReset); }
        public void BtnReset()
        {
            SelectedShelfTypeItem = null;
            ShelfCode = string.Empty;
        }

        public ICommand BtnSearchCommand { get => new DelegateCommand(BtnSearchReset); }
        public void BtnSearchReset()
        {
            BtnSearch(true);
        }

        public void BtnSearch(bool IsPageReset = true)
        {
            if (CurrentPage == 0 || IsPageReset)
            {
                CurrentPage = 1;
                return;
            }
            #region 调用接口获取数据
            var dia = Dialog.Show(new TextDialog());
            try
            {
                var body = new GetShelvesRequest()
                {
                    ShelfTypeId = SelectedShelfTypeItem == null ? 0 : SelectedShelfTypeItem.Id,
                    ShelfCode = ShelfCode,
                    UserName = LocalStatic.CurrentUser,
                    DeviceType = LocalFile.Config.DeviceType,
                    PageNumber = CurrentPage,
                    PageSize = 10,
                };
                var Result = ApiHelp.GetDataFromHttp<PageQueryResponse<ShelfInfoModel>>(LocalFile.Config.ApiIpHost + "storeInfo/getShelves", body, "POST");
                if (Result != null && Result.Data != null && Result.Data.Lists != null)
                {
                    DataGridItemSource = Result.Data.Lists;
                    MaxPage = Result.Data.MaxPage;
                    TotalCount = Result.Data.TotalCount;
                }
            }
            catch (Exception ex)
            {
                Growl.Error("加载数据失败：" + ex.Message);
            }
            finally
            {
                dia.Close();
            }
            #endregion
        }

        /// <summary>
        /// 物料新增操作
        /// </summary>
        public ICommand BtnAddCommand { get => new DelegateCommand(BtnAdd); }
        public async void BtnAdd()
        {
            var addView = new ShelfInfoAddOrUpdateView("新增货架");
            addView.ShowDialog();
            if (addView.DialogResult == true)
            {
                //添加或修改成功后重新查询
                CurrentPage = 1;
            }
        }

        public ICommand BtnEditCommand { get => new DelegateCommand(BtnEdit); }
        public async void BtnEdit()
        {
            //查询勾选的第一个数据
            var shelfInfo = DataGridItemSource?.Where(t => t.IsSelected == true).FirstOrDefault();
            if (shelfInfo == null)
            {
                Growl.Warning("请选择需要修改的数据！");
            }
            else
            {
                var addView = new ShelfInfoAddOrUpdateView("修改货架", shelfInfo);
                addView.ShowDialog();
                if (addView.DialogResult == true)
                {
                    CurrentPage = 1;
                }
            }
        }

        public ICommand BtnDeleteCommand { get => new DelegateCommand(BtnDelete); }
        public void BtnDelete()
        {
            //查询勾选的第一个数据
            var shelfInfo = DataGridItemSource?.Where(t => t.IsSelected == true).FirstOrDefault();
            if (shelfInfo == null)
            {
                Growl.Warning("请选择需要删除的数据！");
            }
            else
            {
                var body = new AddShelfInfoRequest<ShelfInfoModel>()
                {
                    UserName = LocalStatic.CurrentUser,
                    DeviceType = LocalFile.Config.DeviceType,
                    ShelfInfo = shelfInfo,
                    AddOrUpdate = AddOrUpdate.Delete
                };
                var Result = ApiHelp.GetDataFromHttp<ResponseBase<object>>(LocalFile.Config.ApiIpHost + "storeInfo/addOrUpdateShelfInfo", body, "POST");
                if (Result != null && Result.Code == 200)
                {
                    Growl.Success("删除成功！");
                    CurrentPage = 1;
                }
                else
                {
                    Growl.Error($"{Result?.Message?.ToString()}");
                }
            }
        }
        #endregion

        #region PageOperation 分页操作
        private int currentPage;
        public int CurrentPage
        {
            get { return currentPage; }
            set
            {
                SetProperty(ref currentPage, value);
                BtnSearch(false);
            }
        }

        private int maxPage;
        public int MaxPage
        {
            get { return maxPage; }
            set { SetProperty(ref maxPage, value); }
        }

        //总数量
        private int totalCount;
        public int TotalCount
        {
            get { return totalCount; }
            set { SetProperty(ref totalCount, value); }
        }

        public ICommand BtnFirstPageCommand { get => new DelegateCommand(BtnFirstPage); }
        public void BtnFirstPage()
        {
            CurrentPage = 1;
        }

        public ICommand BtnPrePageCommand { get => new DelegateCommand(BtnPrePage); }
        public void BtnPrePage()
        {
            if (CurrentPage > 1)
            {
                CurrentPage--;
            }
        }

        public ICommand BtnNextPageCommand { get => new DelegateCommand(BtnNextPage); }
        public void BtnNextPage()
        {
            if (CurrentPage < MaxPage)
            {
                CurrentPage++;
            }
        }

        public ICommand BtnLastPageCommand { get => new DelegateCommand(BtnLastPage); }
        public void BtnLastPage()
        {
            if (CurrentPage != MaxPage)
            {
                CurrentPage = MaxPage;
            }
        }
        #endregion
    }
}
